/*=============================================================================

    This file is part of FLINT.

    FLINT is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FLINT is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FLINT; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

=============================================================================*/
/******************************************************************************

    Copyright (C) 2011 Fredrik Johansson

******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <mpir.h>
#include "flint.h"
#include "fmpq.h"
#include "fmpq_mat.h"

int
main(void)
{
    int i;
    flint_rand_t state;
    flint_randinit(state);

    printf("det....");
    fflush(stdout);

    for (i = 0; i < 1000; i++)
    {
        fmpq_mat_t A, B, C;
        fmpq_t a, b, ab, c;

        long n, bits;

        n = n_randint(state, 10);
        bits = 1 + n_randint(state, 100);

        fmpq_mat_init(A, n, n);
        fmpq_mat_init(B, n, n);
        fmpq_mat_init(C, n, n);

        fmpq_init(a);
        fmpq_init(b);
        fmpq_init(ab);
        fmpq_init(c);

        fmpq_mat_randtest(A, state, bits);
        fmpq_mat_randtest(B, state, bits);
        fmpq_mat_mul(C, A, B);

        fmpq_mat_det(a, A);
        fmpq_mat_det(b, B);
        fmpq_mat_det(c, C);

        fmpq_mul(ab, a, b);

        if (!fmpq_equal(ab, c))
        {
            printf("FAIL!\n");
            printf("A:\n");
            fmpq_mat_print(A);
            printf("B:\n");
            fmpq_mat_print(B);
            printf("C:\n");
            fmpq_mat_print(C);
            printf("\ndet(A):\n");
            fmpq_print(a);
            printf("\ndet(B):\n");
            fmpq_print(b);
            printf("\ndet(C):\n");
            fmpq_print(c);
            abort();
        }

        fmpq_clear(a);
        fmpq_clear(b);
        fmpq_clear(ab);
        fmpq_clear(c);

        fmpq_mat_clear(A);
        fmpq_mat_clear(B);
        fmpq_mat_clear(C);
    }

    flint_randclear(state);

    _fmpz_cleanup();
    printf("PASS\n");
    return 0;
}